//
//  PopupController.h
//  browser
//
//  Created by knowles on 11-02-11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "PopupController_Protocol.h"

#import "PopupLocation_Protocol.h"

@class URLBarView;

#import "TabsListView.h"
#import "UICBothOrientationsViewController.h"
#import "URLEntryView.h"
#import "UICModalPresenterView.h"
#import "BMUIBookmarkEditViewController.h"
#import "TabHistoryViewController.h"
#import "BMUIBookmarkListViewController.h"
#import "SearchEngineListViewController.h"
#import "SHKLiveItem.h"
#import "UICStackViewController.h"
#import "GeneralDelegate.h"
#import "TZPopoverControllerWrapper.h"
#import "WEPopoverController.h"
#import "WEPopoverController_TZPopoverController.h"

@protocol PopupControllerStackDelegate

- (void) showURL;
- (void) showSearch;
- (void) showTabs;
- (void) showBookmarks;
- (void) showBookmarkFolders:(NSArray *)folders;

- (void) saveState;

@end

@protocol PopupControllerDelegate

- (UIViewController *) viewController;
- (NSObject<
	URLEntryViewDelegate,
	URLSuggestionsDelegate,
	UICPopupViewControllerDelegate,
	URLBarViewDelegate
	> *) browserDelegate;
- (UIView *) primaryView;

- (id<
	SearchEngineListViewControllerDelegate
	>) viewArrangementDelegate;

- (id<PopupControllerStackDelegate>) stackDelegate;

@property (readonly) UIView * backButton;
@property (readonly) UIView * forwardButton;
@property (readonly) UIView * tabsButton;
@property (readonly) UIView * tabOptionsView;
@property (readonly) UIView * actionsButton;
@property (readonly) UIView * searchButton;
@property (readonly) UIView * bookmarksButton;
@property (readonly) URLBarView * urlBarView;

- (void) setSearchOptionsViewVisible:(BOOL)visible animated:(BOOL)animated delegate:(id)delegate;
- (void) resetAfterPopoverDismissedAnimated:(BOOL)animated;

- (BOOL) isLandscape;
- (BOOL) isFullscreen;

- (UICStackViewController *) stackViewController;
- (void) setStackVisible:(BOOL)visible animated:(BOOL)animated;

- (NSArray *) passthroughViews;

@end

@interface PopupController : NSObject <PopupController,TZPopoverControllerWrapperDelegate,WEPopoverControllerDelegate,ModalPresenterViewDelegate> {
  

}

@property (nonatomic, weak) id<PopupControllerDelegate> delegate;

@property (nonatomic, strong) id<TZPopoverController> popoverController;
@property (nonatomic, strong) UICModalPresenterView *modalPresenterView;

@property (nonatomic, strong) TabsListView * tabsListView;
@property (nonatomic, strong) URLSuggestionsViewController * urlSuggestionsViewController;

@property (nonatomic, strong) IBOutlet UIBarButtonItem *tabsThumbnailBarButton;
@property (nonatomic, strong) IBOutlet UIBarButtonItem *tabsListBarButton;
@property (nonatomic, strong) IBOutlet UIBarButtonItem *tabsAddTabBarButton;
@property (nonatomic, strong) IBOutlet UIBarButtonItem *tabsSortButton;
@property (nonatomic, strong) IBOutlet UIBarButtonItem *tabsMoreButton;

@property (nonatomic, strong) IBOutlet UIBarButtonItem *doneBarButton;
@property (nonatomic, strong) IBOutlet UIBarButtonItem *doneStackBarButton;

@property (nonatomic, strong) id<PopupLocation> activeLocation;

#pragma mark -
#pragma mark Initialization

- (void) initialize;

#pragma mark -
#pragma mark Popup Presentation

- (UIViewController *)addNavControllerIfNeededToViewController:(UIViewController *)viewController addDoneButton:(BOOL)addDoneButton;

- (void)presentPopupViewController:(UIViewController *)controller fromLocation:(id<PopupLocation>)location animated:(BOOL)animated;

- (BOOL)shouldJustClosePopupBasedOnLocation:(id<PopupLocation>)location;

- (id<PopupLocation>)defaultFullscreenPopupLocation;

#pragma mark -
#pragma mark Modal Popups

- (void)presentModalPopupViewController:(UIViewController *)controller animated:(BOOL)animated;

#pragma mark -
#pragma mark Popover Popups

- (void)preparePopoverViewController:(UIViewController *)controller;

#pragma mark -
#pragma mark IBActions

- (IBAction) touchAddTabButton:(id)sender;
//- (IBAction) touchSidebarButton:(id)sender;
- (IBAction) touchTabThumbnailsButton:(id)sender;
- (IBAction) touchTabListButton:(id)sender;
- (IBAction) touchTabSortButton:(id)sender;
- (IBAction) touchTabMoreButton:(id)sender;
- (IBAction) touchDoneButton:(id)sender;
- (IBAction) touchDoneStackButton:(id)sender;

@end
