
if (typeof Thinkbitz === 'undefined') var Thinkbitz = {};

if (typeof Thinkbitz.FindOnPageTools === 'undefined') Thinkbitz.FindOnPageTools = new (function ()
{
	// We're using a global variable to store the number of occurrences
	this.searchResultCount = 0;
	this.latestKeyword = '';
	this.latestIndex = -1;
	this.foundSpans = [];

	// helper function, recursively searches in elements and their child nodes
	this.highlightAllOccurencesOfStringForElement = function(element,keyword) {
	
		if (element) {
			if (element.nodeType == 3) {        // Text node
				while (true) {
					var value = element.nodeValue;  // Search for keyword in text node
					var idx = value.toLowerCase().indexOf(keyword);
					
					if (idx < 0) break;             // not found, abort
					
					var span = document.createElement("span");
					var text = document.createTextNode(value.substr(idx,keyword.length));
					
					span.appendChild(text);
					span.setAttribute("class","Thinkbitz_FindOnPageTools_Highlight");
					span.style.backgroundColor="yellow";
					span.style.color="black";
					
					this.foundSpans[this.foundSpans.length] = span;				
					
					text = document.createTextNode(value.substr(idx+keyword.length));
					element.deleteData(idx, value.length - idx);
					var next = element.nextSibling;
					element.parentNode.insertBefore(span, next);
					element.parentNode.insertBefore(text, next);
					element = text;
					
					this.searchResultCount++;	// update the counter
				}
			} else if (element.nodeType == 1) { // Element node
				if (element.style.display != "none" && element.nodeName.toLowerCase() != 'select' && element.nodeName.toLowerCase() != 'script') {
					for (var i=element.childNodes.length-1; i>=0; i--) {
						this.highlightAllOccurencesOfStringForElement(element.childNodes[i],keyword);
					}
				}
			}
		}
	};

	// the main entry point to start the search
	this.highlightString = function(keyword) {
		
		if (this.latestKeyword != keyword)
		{
			this.removeAllHighlights();
			
			if (keyword != '')
			{
				this.highlightAllOccurencesOfStringForElement(document.body, keyword.toLowerCase());
				this.latestKeyword = keyword;
			}
			
		}

		if (keyword != '')
		{
			if (this.foundSpans.length > 0)
			{
				var index;
				var currentElement;
				
				if (this.latestIndex != -1)
				{
					index = this.foundSpans.length - this.latestIndex - 1;
					currentElement = this.foundSpans[index];
					currentElement.style.backgroundColor = "yellow";
				}
				
				this.latestIndex++;
				
				if (this.latestIndex >= this.foundSpans.length)
				{
					this.latestIndex = 0;
				}
				
				index = this.foundSpans.length - this.latestIndex - 1; // we actually want to go backwards through the array
				currentElement = this.foundSpans[index];
				currentElement.style.backgroundColor = "red";
					
				this.scrollToElement(currentElement);
			}
		}
	};

	// helper function, recursively removes the highlights in elements and their childs
	this.removeAllHighlightsForElement = function(element) {
		if (element) {
			if (element.nodeType == 1) {
				if (element.getAttribute("class") == "Thinkbitz_FindOnPageTools_Highlight") {
					var text = element.removeChild(element.firstChild);
					element.parentNode.insertBefore(text,element);
					element.parentNode.removeChild(element);
					return true;
				} else {
					var normalize = false;
					for (var i=element.childNodes.length-1; i>=0; i--) {
						if (this.removeAllHighlightsForElement(element.childNodes[i])) {
							normalize = true;
						}
					}
					if (normalize) {
						element.normalize();
					}
				}
			}
		}
		return false;
	};
	
	// the main entry point to remove the highlights
	this.removeAllHighlights = function() {
		
		this.latestKeyword = '';
		this.latestIndex = -1;
		this.foundSpans = [];
		
		this.searchResultCount = 0;
		this.removeAllHighlightsForElement(document.body);
	};

	this.scrollToElement = function(theElement){
		
		var selectedPosX = 0;
		var selectedPosY = 0;
		
		while(theElement != null){
			selectedPosX += theElement.offsetLeft;
			selectedPosY += theElement.offsetTop;
			theElement = theElement.offsetParent;
		}
		
		window.scrollTo(selectedPosX,selectedPosY);
	};
	
})();

