
if (typeof Thinkbitz === 'undefined') var Thinkbitz = {};

if (typeof Thinkbitz.LinkDetectionTools === 'undefined') Thinkbitz.LinkDetectionTools = new (function ()
{

	this.getLinkAtPosition = function(x,y,webViewWidth,webViewHeight)
	{
		try
		{
			var convertedPoint = this.convertPoint(x,y,webViewWidth,webViewHeight);
			var convertedX = convertedPoint.x;
			var convertedY = convertedPoint.y;
			
			var that = this;
			var link = this.searchForItem(convertedX,convertedY,function(element) { return that.findLink(element); });
		}
		catch(e)
		{
			alert(e);
		}
		
		return link;
		
	};
	
	this.getImageAtPosition = function(x,y,webViewWidth,webViewHeight)
	{
		try
		{
			var convertedPoint = this.convertPoint(x,y,webViewWidth,webViewHeight);
			var convertedX = convertedPoint.x;
			var convertedY = convertedPoint.y;
			
			var that = this;
			var image = this.searchForItem(convertedX,convertedY,function(element) { return that.findImage(element); });
		}
		catch(e)
		{
			alert(e);
		}
		
		if (image != null)
		{
		
			var returnValue = {};
			returnValue['image'] = image.src;
			returnValue['alt'] = image.title;
			
			return JSON.stringify(returnValue);
		
		}
		else
		{
			return null;
		}
	};
	
	this.searchForItem = function(x,y,findFunction)
	{
		var element = this.elementFromPoint(x,y);
		var item = findFunction(element);
				
		if (item === null)
		{
			// if we don't find it, hunt around a bit to see if the finger's a bit off...
			for (var fudge=0; fudge<20 && item===null; fudge++)
			{
				var fudgeCoordinates = this.fudgeCoordinates(x,y,fudge);
					
				for (var index=0; index<fudgeCoordinates.length && item===null; index++)
				{
					var coords = fudgeCoordinates[index];
					
					element = this.elementFromPoint(coords.x, coords.y);
					item = findFunction(element);
				}
				
			}
		}
		
		return item;
	}
	
	this.elementFromPoint = function(x,y)
	{
		if (!this.scrollHasBeenChecked)
		{
			var scroll;
			if((scroll = window.scrollY) >0)
			{
				this.isRelative = (document.elementFromPoint(0, scroll + window.innerHeight -1) == null);
				this.scrollHasBeenChecked = true;
			}
			else if((scroll = window.scrollX) >0)
			{
				this.isRelative = (document.elementFromPoint(scroll + window.innerWidth -1, 0) == null);
				this.scrollHasBeenChecked = true;
			}
		}
		
		if(!this.isRelative)
		{
			x += window.scrollX;
			y += window.scrollY;
		}
		
		return document.elementFromPoint(x,y);
	};
	
	this.convertPoint = function(x,y,webViewWidth,webViewHeight)
	{
		var conversionX = window.innerWidth / webViewWidth;
		var conversionY = window.innerHeight / webViewHeight;
		
		var convertedX = (conversionX * x);
		var convertedY = (conversionY * y);
		
		return { x:convertedX, y:convertedY };
	};
	
	this.findLink = function(element)
	{
		if (element)
		{
			if (element.nodeName.toLowerCase() == "a")
			{
				return element.href;
			}
			
			if (element.parentNode)
			{
				return this.findLink(element.parentNode);
			}
		}
		
		return null;
	};
	
	this.findImage = function(element)
	{
		if (element)
		{
			if (element.nodeName.toLowerCase() == "img")
			{
				return element;
			}

			if (element.parentNode)
			{
				return this.findImage(element.parentNode);
			}
			else
			{
				return null;
			}
		}

	};
	
	this.fudgeCoordinates = function(convertedX,convertedY,fudge)
	{
		var fudgeCoordinates = [
			{ x:convertedX, y:convertedY + fudge},
			{ x:convertedX, y:convertedY - fudge},
			{ x:convertedX + fudge, y:convertedY },
			{ x:convertedX - fudge, y:convertedY },
			{ x:convertedX + fudge, y:convertedY + fudge},
			{ x:convertedX - fudge, y:convertedY + fudge},
			{ x:convertedX + fudge, y:convertedY - fudge},
			{ x:convertedX - fudge, y:convertedY - fudge}
			];
			
		return fudgeCoordinates;
	};
	
	this.addDivAtPoint = function(x,y)
	{
		// For test purposed; note: won't display properly on non-relative pages (that need scroll compensation)
		var div = document.createElement('div');
		
		div.style.backgroundColor = "green";
		div.style.left = x + "px";
		div.style.top = y + "px";
		div.style.width = 10 + "px";
		div.style.height = 10 + "px";
		div.style.position = "absolute";
		
		document.body.appendChild(div);
	}

})();









